#/usr/bin/env perl

use strict;
use warnings;
use JSON::XS;
use Curses::UI;
use Curses::UI::Common;

my %CONF = (
	"working_directory","/usr/share/linkat-sc-configurador",
	"config_directory","/usr/share/linkat-sc-configurador/conf",
	"config_file",		"config.json",
	"set_variable",		"set_variable.sh",	
	
	"bg", 				"blue",
	"fg", 				"white",
	"bfg", 				"white",
	
	"buttons_width",	"40",
	
	"window_title",		"Linkat: Configuració del servidor de comunicacions"
);


# from file content
local $/;
open( my $fh, '<', $CONF{working_directory} . "/" . $CONF{config_file});
my $json_text   = <$fh>;
close($fh);

my $typologies = JSON::XS->new->utf8->decode( $json_text );
my $menu;
my $win;
my @buttons;

foreach my $typo (@{$typologies}) {
	push @buttons, {		
		-label => $typo->{name},
		-onpress => \&edit_typology
	};
}

my $cui = new Curses::UI( -color_support => 1 );


sub create_window {
	$win = $cui->add(
		'win', 'Window',
		-border	=> 1,
		-bfg  	=> $CONF{bfg},
		-bbg  	=> $CONF{bg},
		-bg 	=> $CONF{bg},
		-fg 	=> $CONF{fg}
	);

	my $title = $win->add(
		'title', 'Label',
		-text	=> $CONF{window_title},
		-bold   => 1,
		-y      => 0,
		-x      => 1
	);
}

sub create_menu {
	create_window();

	my $menu_x = ($win->width() - $CONF{buttons_width}) / 2;
	my $menu_y = ($win->height() - (4 + scalar @buttons)) / 2;

	$menu = $win->add(
		'menu', 'Buttonbox',
		-buttons   			=> [@buttons],
		-vertical   		=> 1,
		-buttonalignment	=> 'middle',
		-title      		=> 'Tipologia del centre',
		-border     		=> 1,
		-bbg  				=> $CONF{bg},
		-tbg 				=> $CONF{fg},
		-tfg 				=> $CONF{bg},
		-ipad 				=> 1,
		-width      		=> $CONF{buttons_width},
		-x  				=> $menu_x,
		-y  				=> $menu_y,
	);
	$menu->focus();
}

sub edit_typology {
	my $typology = $buttons[$menu->get()]{-label};
	my $current_y = 2;
	my $current_x = 1;
	my $max_y = 30; 
	my $label_width = 28;
	my $input_width = 18;
	my $i = 0;
	my $typology_id = $menu->get();
	
	$cui->delete('win');
	
	create_window();
	
	foreach my $variable (@{$$typologies[$typology_id]->{variables}}) {
		if($variable->{editable}){
			$win->add(
				"d$i", "TextEntry", 
				-border => 0,
				-bbg  	=> $CONF{bg},
				-x => $current_x ,
				-y => $current_y + 1,
				-width => $label_width,
				-text => $variable->{label},
				-focusable => 0,
				-readonly => 1
			);
			$win->add(
				"input-$i", "TextEntry", 
				-border => 1,
				-bbg  	=> $CONF{bg},
				-y => $current_y,
				-x => $current_x + $label_width ,
				-text => $variable->{value},
				-width => $input_width
			);
			$current_y += 3;

			if ($current_y > $max_y) {
				$current_y = 2;
				$current_x = 50;
			}
		}
		$i = $i + 1; 
	}


	my $button = $win->add(
		"addbutton", "Buttonbox",
		-buttons => [ 	
		{ 
			-label => "[ APLICA ]",
			-onpress => \&apply_typology
		} ,
		{ 
			-label => "[ ENRERE ]",
			-onpress => \&restart
		}  
		],
		-y => $current_y + 2, 
		-x => $current_x
	);
	$button->focus();
}

sub apply_typology {
	my $typology = $buttons[$menu->get()]{-label};
	my $typology_id = $menu->get();
	my $i = 0;

	foreach my $variable (@{$$typologies[$typology_id]->{variables}}) {
		if($variable->{editable}){
			$variable->{value} = $win->getobj("input-$i")->get();
		}
		$i = $i + 1; 
	}
	
	my $yes = $cui->dialog(
    	-message => "Vols aplicar la configuracio " . $typology . "?",
        -buttons => [
        	{-label => "SI",-value => 1}, 
        	{-label => "NO",-value => 0}
        ],        
	    -bfg  	=> $CONF{bfg},
	    -bbg  	=> $CONF{bg},
	    -bg 	=> $CONF{bg},  
    );    

    if ($yes) {
    	$cui->delete("win");
    	$cui->mainloopExit;
    	system("reset");
    	do_backup("pre-configurador");
	copy_config_files($menu->get());
    	set_variables($menu->get());
	execute_commands($menu->get());
    	set_services($menu->get());
    	do_backup("post-configurador"); 
    }
}

sub restart {
	$cui->delete('win');
	create_menu();

}

sub exit {    
	my $yes = $cui->dialog(
   	-message => "Segur que vols sortir sense aplicar?",
        -buttons => [
        	{
        		-label => "SI",
        		-value => 1
        	}, 
        	{
        		-label => "NO",
        		-value => 0
        	}
        ],        
	    -bfg  	=> $CONF{bfg},
	    -bbg  	=> $CONF{bg},
	    -bg 	=> $CONF{bg},
    );    

    if ($yes) {
        exit 0;
    }
}

sub execute_commands{
	my $typology_id = shift;

	foreach my $command (@{$$typologies[$typology_id]->{commands}}) {
		system($command);
	}
}

sub do_backup {
	system(
		"/usr/bin/sc-backup", 
		shift
	);
}

sub copy_config_files {
	my $typology_id = $menu->get();
	foreach my $directory (@{$$typologies[$typology_id]->{config}}) {
		system("cp -rpv $CONF{config_directory}/$directory/* /");
	}
}

sub set_variables {
	my $typology_id = $menu->get();
	foreach my $variable (@{$$typologies[$typology_id]->{variables}}) {
		system(
			"$CONF{working_directory}/$CONF{set_variable}", 
			$variable->{file}, 
			$variable->{name}, 
			$variable->{value}, 
			$variable->{wrapper}
		);
	}
}

sub set_services {
	my $typology_id = $menu->get();
	foreach my $service (@{$$typologies[$typology_id]->{services}}) {
		if ($service->{value}) {
			system("chkconfig $service->{name} on");
		} else {
			system("chkconfig $service->{name} off");	
		}
	}
	
	foreach my $service (@{$$typologies[$typology_id]->{services}}) {		
		if ($service->{value}) {
			system("service $service->{name} restart");
		} else {
			system("service $service->{name} stop");
		}
	}
}

create_menu();
$cui->set_binding( \&exit, CUI_ESCAPE);
$cui->mainloop;
